#ifndef EDMONDSKARP_H_
#define EDMONDSKARP_H_

#include "graph.hpp"
#include <vector>
#include <map>

class edmonds_karp {
private:
	const graph &g;

	std::map<std::pair<node_id, node_id>, int> indexed_edges;

	node_id source;
	node_id sink;

	int flow_value;
	std::vector<int> flow;

	/**
	 * Indexes edges of the graph
	 * Every edge in the graph is mapped to its unique id
	 */
	void index_edges();
	/**
	 * Performs a breadth-first search on the graph from the source node to find an augmenting path to the sink node respecting the flow values
	 * @param residFlow The residual flow in the network.
	 * @param pred Used to store the path from the source to the sink.
	 * @return The gain in terms of flow.
	 */
	int bfs(std::vector<int> &resid_flow, std::vector<unsigned int> &pred) const;

public:
	/**
	 * Constructs an instance of the EdmondsKarp algorithm for the given graph, source and sink
	 * @param graph The graph.
	 * @param source The source node.
	 * @param sink The sink node.
	 */
	 edmonds_karp(const graph &g, node_id source, node_id sink);

	/**
	 * Computes the maximum flow, executes the EdmondsKarp algorithm.
	 * For unweighted, undirected Graphs.
	 */
	void run();

	/**
	 * Returns the value of the maximum flow from source to sink.
	 *
	 * @return The maximum flow value
	 */
	int get_max_flow() const;
};

#endif /* EDMONDSKARP_H_ */